//  **** BEGIN LICENSE BLOCK ****
//  Copyright(c) 2005-2006 Adam Judson
//
//
//  PhotoMap
//  - display user contributed photos in a map
//
//  This program is free software; you can redistribute it and/or modify it under
//  the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
//  more details.
//
//  You should have received a copy of the GNU General Public License along with
//  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
//  Place, Suite 330, Boston, MA 02111-1307 USA
//
//  **** END LICENSE BLOCK ****


function PhotoMapController(gui, location) {
   this.init(gui, location);
}

PhotoMapController.prototype = {
   preferences : new PhotoMapPreferences(),

   // Initialisation and termination functions
   init : function(gui, location) {
      this.photoMode = "view"
      this.gui = gui;
      this.photo = new PhotoMapPhotoAgent(this);
      this.map   = new PhotoMapMapAgent(this);
      if (location) {
         this.startingPoint = location;
      } else {
         this.startingPoint = this.preferences.getHome();
      }
      this.map.createMap(this.gui.getMapIFrame());
   },
   
   shutdown : function() {
      this.photo.shutdown();
      this.map.shutdown();
   },

   launchOptions : function() {
      window.openDialog("chrome://photomap/content/PhotoMapOptions.xul",
                        "PhotoMapOptions",
                        "chrome,dialog=yes");
   },

   goHome : function() {
      var point = this.preferences.getHome();
      this.map.setLocation(point, this.preferences.getHomeZoom());
   },

   setHome : function() {
      var point = this.map.getLocation();
      var zoom = this.map.getZoom();
      this.preferences.setHome(point);
      this.preferences.setHomeZoom(zoom);
      this.updateTransactionTotals();
   },

   mapInit : function() {
      this.goHome();
      this.setLocation(this.startingPoint);
      delete this.startingPoint;
   },

   mapMoved : function(force) {
      // this is called by the map object whenever we move or zoom
      var loc = this.map.getLocation();
      var bounds = this.map.getBounds();
      var zoom = this.map.getZoom();
      this.gui.setMapInfo(loc, bounds, zoom);

      this.updateMarkers();

      if (this.isView()) {
         // let the photo controller know we have a new target
         this.photo.locationChanged(loc, bounds, zoom, force);
      }
      this.updateTransactionTotals();
   },

   refresh : function() {
      this.mapMoved(true);
   },

   updateMarkers : function() {
      this.updateAllCurrentMarkers();   
      if (this.isEdit()) {
         this.map.updateCenterMarker();
      }   
   },


   markerUpdate : function(key) {
      this.updateMarkersByKey(key);
      // change this to update debug info or something
      // and put all of that stuff on a div we can hide/show
      this.updateTransactionTotals();
   },


   updateMarkersByKey : function(key) {
      var mapThings = this.photo.getByKey(key);
      // we really only need to call this once, because the key is a map location, and we don't want
      // more than one marker in the same place
      for (thingKey in mapThings) {
         // map should deal with muliple calls to add marker,
         this.map.addMarker(mapThings[thingKey]);
         break;
      }
   },

   markerSelected : function(lat, lon) {
      // find the image
      var imageSet = this.photo.getByLatLon(lat, lon);
      if (imageSet) {
         this.gui.setPhotos(imageSet);
      }
   },

   updateAllCurrentMarkers : function() {
      // this should probably retrieve a list of keys
      var mapThings = this.photo.getAll();
      if (this.photoMode == "view" && mapThings) {
         for (thingKey in mapThings) {
            this.updateMarkersByKey(thingKey);
         }
      }
   },

   isEdit : function() {
      return this.photoMode == "edit";
   },

   isView : function() {
      return !this.isEdit();
   },

   setPhotoViewMode : function() {
      this.photoMode = "view";
      this.map.clearMarkers();
      this.mapMoved();
   },

   setPhotoEditMode : function() {
      this.photoMode = "edit";
      this.photo.stop();
      this.map.clearMarkers();
      this.mapMoved();

      this.gui.setNeedsAuthentication(this.photo.needsAuthentication());
   },

   setLocation : function(loc) {
      this.map.setLocation(loc);
      if (loc.zoom) {
         this.map.setLocation(loc, loc.zoom);
      }
      if (loc.type) {
         this.map.setMapType(loc.type);
      }
   },

   gotoPhoto : function(photo) {
      if (photo) {
         var loc = {lat : photo.lat, lon : photo.lon};
         this.setLocation(loc);
      }
   },

   getEditInfo : function() {
      var loc = this.map.getLocation();
      var lat = loc.lat;;
      var lon = loc.lon;
      var type = this.gui.getEditType();
      var zoom = this.map.getZoom();
      var season = this.gui.getEditSeason();
      var tod = this.gui.getEditTOD();
      var pov = this.gui.getEditPOV();
      var mapType = this.map.getType();
      var other = this.gui.getEditOtherTags();

      return new EditInfo(lat, lon, type, zoom, season, tod, pov, mapType, other);
   },

   updateTags : function() {
      // move this to photo agent -> object
      var tags = this.photo.getTags(this.getEditInfo());
      this.gui.setEditOutputTags(tags);
   },

   photoAuthenticate : function() {
      this.photo.authenticate();
   },

   photoDoneAuthenticate : function() {
      this.photo.doneAuthenticate();
   },

   openWindow : function(url) {
      this.gui.openWindow(url);
   },

   getNewAccountUrl : function() {
      return this.photo.getNewAccountUrl();
   },

   authenticationSuccess : function() {
      this.gui.setAuthenticated();
   },

   getPhotoUserName : function() {
      return this.photo.getUserName();
   },

   getPhotoTotalPictures : function() {
      return this.photo.getTotalPictures();
   },

   photoLogout : function() {
      this.photo.logout();
   },

   loadFile : function() {
      var photoFile = PhotoMapUtils.getImageFile("Select a file:");
      if (photoFile) {
         this.photoFile = photoFile;
         this.gui.setPhotoFile(photoFile.target);
      }
   },

   upload : function() {
      // window.alert("Add some checks here");
      if (this.photoFile) {
         var editInfo = this.getEditInfo();
         editInfo.file = this.photoFile;
         this.photo.upload(editInfo, new Callback(this, this.doneUpload));
      }
   },

   doneUpload : function(wasError) {
      this.gui.doneUpload(wasError);
   },

   launchHelp : function() {
      this.gui.openNewTab("http://photomap.mozdev.org/help.html");
   },

   copyTags : function(tags) {
      PhotoMapUtils.copyToClipboard(tags);
   },

   // this is evil
   updateTransactionTotals : function() {
      var transactionsPerSecond = BaseFlickrPhotoTransaction.transactionsPerSecond();
      this.gui.updateTransactionTotal(transactionsPerSecond + " txn/sec");
   },

   copyLinkText : function() {
      var editInfo = this.getEditInfo();
      var link = editInfo.uri();
      PhotoMapUtils.copyToClipboard(link);
   }
}

function EditInfo(lat, lon, type, zoom, season, tod, pov, maptype, other) {
   this.init(lat, lon, type, zoom, season, tod, pov, maptype, other);
}

EditInfo.prototype = {
   init : function(lat, lon, type, zoom, season, tod, pov, maptype, other) {
      this.lat = lat;
      this.lon = lon;
      this.type = type;
      this.zoom = zoom;
      this.season = season;
      this.tod = tod;
      this.pov = pov;
      this.maptype = maptype;
      this.otherInfo = other;
   },

   uri : function() {
      return "photomap://map?lat=" + this.lat +"&long=" + this.lon + "&zoom=" + this.zoom + "&type=" + this.maptype;
   }
}

